#!/usr/bin/env python3
import argparse
import datetime
import json
import pathlib
import sys

from feed_manager import consumer


def main():
    """Simple script to generate a daily feed."""
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "-i",
        "--input-dir",
        dest="input_dir",
        type=str,
        default=None,
        required=True,
        help="the local feed",
    )
    parser.add_argument(
        "-t",
        "--attribute-type",
        dest="attribute_type",
        type=str,
        default=None,
        help="the attribute type",
    )
    parser.add_argument(
        "-d",
        "--day-delta",
        dest="day_delta",
        type=int,
        default=52,
        help="the look back in terms of days",
    )
    args = parser.parse_args()

    pathlib.Path(args.input_dir).mkdir(parents=True, exist_ok=True)
    since_date_object = datetime.datetime.utcnow() - datetime.timedelta(days=args.day_delta)
    feed_consumer = consumer.LocalFeedConsumer(args.input_dir)
    indicators = feed_consumer.get_items_since(
        date_object=since_date_object,
        attribute_type=args.attribute_type,
    )

    print(f"Fetching items since {since_date_object}")
    for indicator in indicators:
        print(json.dumps(indicator, indent=True))
    return 0


if __name__ == "__main__":
    sys.exit(main())



import itertools
import feed_manager

from typing import Any
from typing import Dict
from typing import List
from typing import Optional
from typing import Tuple
from typing import Union


try:
    import pymisp
except ImportError as ie:
    pymisp = None
    feed_manager.print_dependency_error_and_raise(ie)


class TagUtils:
    """Class with utility methods to handle tags."""

    OBJECT_NAME_TO_ATTRIBUTE_TAG = {
        "file": "filename",
        "network-profile": "text",
        "sandbox-report": "score",
        "atp-report": "score",
    }

    @classmethod
    def __workaround_bug(cls, object_attribute: pymisp.MISPAttribute) -> pymisp.MISPAttribute:
        """Workaround the current pymisp bug where object attributes have mis-initialized tags."""
        if object_attribute.tags:
            return object_attribute
        if not hasattr(object_attribute, "AttributeTag"):
            return object_attribute
        tags = []
        try:
            for attribute_tag in object_attribute.AttributeTag:
                tag = attribute_tag["Tag"]
                tag_object = pymisp.MISPTag()
                tag_object.from_dict(**tag)
                tags.append(tag_object)
        except KeyError:
            pass
        object_attribute.tags = tags
        return object_attribute

    @classmethod
    def get_taggable_entity(
        cls,
        entity: Union[pymisp.MISPEvent, pymisp.MISPObject, pymisp.MISPAttribute],
    ) -> Union[pymisp.MISPEvent, pymisp.MISPAttribute]:
        """Get the taggable entity."""
        if isinstance(entity, pymisp.MISPEvent):
            return entity
        elif isinstance(entity, pymisp.MISPAttribute):
            return entity
        else:
            try:
                attribute_type = cls.OBJECT_NAME_TO_ATTRIBUTE_TAG[entity.name]
            except KeyError:
                raise ValueError(f"Can not process object '{entity.name}/{entity.uuid}'")
            try:
                return cls.__workaround_bug(entity.get_attributes_by_relation(attribute_type)[0])
            except IndexError:
                raise ValueError(f"Object '{entity.name}/{entity.uuid}' seems malformed'")

    @classmethod
    def validate_tag(cls, input_object: Union[pymisp.MISPTag, str]) -> pymisp.MISPTag:
        """Validate a tag (whether an object or a string) and return an object."""
        if isinstance(input_object, pymisp.MISPTag):
            return input_object
        else:
            return cls.create_tag(name=input_object)

    @classmethod
    def create_tag(cls, name: str, colour: Optional[str] = None) -> pymisp.MISPTag:
        """Create a tag."""
        tag = pymisp.MISPTag()
        tag.from_dict(
            name=name,
            colour=colour,
        )
        return tag

    @classmethod
    def add_tag_to_object(cls, obj: pymisp.MISPObject, tag: Union[str, pymisp.MISPTag]):
        """Add a tag to an object by choosing a representative attribute."""
        attribute = cls.get_taggable_entity(obj)
        cls.add_tag_to_attribute(attribute, tag)

    @classmethod
    def add_tag_to_attribute(cls, attribute: pymisp.MISPAttribute, tag: Union[str, pymisp.MISPTag]):
        """Add a tag to an attribute."""
        tag = cls.validate_tag(tag)
        attribute.add_tag(tag)

    @classmethod
    def entity_contains_tag(
        cls,
        entity: Union[pymisp.MISPEvent, pymisp.MISPObject, pymisp.MISPAttribute],
        tag_name: str,
    ) -> bool:
        """Whether an entity is tagged with a given tag."""
        if isinstance(entity, pymisp.MISPObject):
            entity = cls.get_taggable_entity(entity)
        for tag in entity.tags:
            if tag.name == tag_name:
                return True
        return False

    @classmethod
    def decode_cluster_tag(
        cls,
        tag: Union[str, pymisp.MISPTag],
    ) -> Tuple[Optional[str], Optional[str]]:
        """Decode a cluster tag."""
        if isinstance(tag, pymisp.MISPTag):
            tag = tag.name
        try:
            category, value = tag.split("=")
            return category, value.strip("\"")
        except (KeyError, IndexError):
            return None, None

    @classmethod
    def get_cluster_tag_value(
        cls,
        tag: Union[str, pymisp.MISPTag],
        category: str = None,
    ) -> Optional[str]:
        """Get the value of a cluster tag."""
        cat, value = cls.decode_cluster_tag(tag)
        if category:
            return value if cat.startswith(category) else None
        else:
            return value


class IndicatorTranslator:
    """Class that translate indicators or telemetry events to MISP objects."""

    DEFAULT_FILE_ATTRIBUTE_CATEGORY = "Payload delivery"
    DEFAULT_NET_ATTRIBUTE_CATEGORY = "Network activity"
    DEFAULT_FILE_NAME = "unknown"
    DEFAULT_WHOIS_ENTRY = "missing"

    @classmethod
    def to_network_attribute(
        cls,
        network_indicator: str,
        tags: Optional[List[Union[str, pymisp.MISPTag]]] = None,
    ) -> pymisp.MISPAttribute:
        """Get a network indicator as attribute."""
        if "://" in network_indicator:
            attribute_type = "url"
        elif len(network_indicator.split(".")) == 4:
            if ":" in network_indicator:
                attribute_type = "ip-dst|port"
            else:
                attribute_type = "ip"
        else:
            attribute_type = "domain"
        net_attribute = pymisp.MISPAttribute()
        net_attribute.from_dict(
            type=attribute_type,
            category=cls.DEFAULT_NET_ATTRIBUTE_CATEGORY,
            value=network_indicator,
        )
        for tag in tags or []:
            TagUtils.add_tag_to_attribute(net_attribute, tag)
        return net_attribute

    @classmethod
    def to_file_attribute(
        cls,
        file_hash,
        attribute_category: str = DEFAULT_FILE_ATTRIBUTE_CATEGORY,
        tags: Optional[List[Union[str, pymisp.MISPTag]]] = None,
    ) -> pymisp.MISPAttribute:
        """Get a file attribute (useful when only a single hash is available)."""
        attribute_type = feed_manager.get_hash_type(file_hash)
        if not attribute_type:
            raise ValueError(f"Invalid hash '{file_hash}'")
        file_attribute = pymisp.MISPAttribute()
        file_attribute.from_dict(
            type=attribute_type,
            category=attribute_category,
            value=file_hash,
        )
        for tag in tags or []:
            TagUtils.add_tag_to_attribute(file_attribute, tag)
        return file_attribute

    @classmethod
    def to_file_object(
        cls,
        file_md5: str,
        file_sha1: str,
        file_sha256: str,
        file_name: Optional[str] = None,
        size: Optional[int] = None,
        mime_type: Optional[str] = None,
        comment: Optional[str] = None,
        attribute_category: str = DEFAULT_FILE_ATTRIBUTE_CATEGORY,
        tags: Optional[List[Union[str, pymisp.MISPTag]]] = None,
    ) -> pymisp.MISPObject:
        """Get a file object (mandatory when having multiple hashes)."""
        file_object = pymisp.MISPObject(name="file")
        file_object.add_attribute(
            "filename",
            value=file_name or cls.DEFAULT_FILE_NAME,
            comment=comment,
        )
        if file_md5:
            file_object.add_attribute(
                "md5",
                value=file_md5,
                category=attribute_category,
            )
        if file_sha1:
            file_object.add_attribute(
                "sha1",
                value=file_sha1,
                category=attribute_category,
            )
        if file_sha256:
            file_object.add_attribute(
                "sha256",
                value=file_sha256,
                category=attribute_category,
            )
        if size:
            file_object.add_attribute(
                "size-in-bytes",
                value=size,
            )
        if mime_type:
            file_object.add_attribute(
                "mimetype",
                value=mime_type,
            )
        for tag in tags or []:
            TagUtils.add_tag_to_object(file_object, tag)
        return file_object

    @classmethod
    def from_contexa_to_objects(
        cls,
        item: Dict[str, Any],
        mitre_attack_technique_id_to_tag: Optional[Dict] = None,
        include_sandbox_result: bool = True,
        include_sandbox_activities: bool = True,
        tags: Optional[List[Union[str, pymisp.MISPTag]]] = None,
    ):
        """Convert a contexa telemetry item into objects."""
        objects = []
        file_object = cls.to_file_object(
            file_md5=item["file.md5"],
            file_sha1=item["file.sha1"],
            file_sha256=item["file.sha256"],
            file_name=item.get("file.name"),
            mime_type=item.get("file.mime_type"),
            size=item.get("file.size"),
        )
        objects.append(file_object)

        sandbox_object = None
        if include_sandbox_result:
            sandbox_object = pymisp.MISPObject(name="sandbox-report")
            sandbox_object.add_attribute("score", item["task.score"])
            sandbox_object.add_attribute("saas-sandbox", "vmware-atp-sandbox")
            sandbox_object.add_attribute("permalink", item["task.portal_url"])
            sandbox_object.add_reference(
                referenced_uuid=file_object.uuid,
                relationship_type="belongs-to",
            )
            objects.append(sandbox_object)

        if sandbox_object and include_sandbox_activities and item.get("analysis.activities"):
            sig_object = pymisp.MISPObject(name="sb-signature")
            for activity in item["analysis.activities"]:
                sig_object.add_attribute("signature", type="text", value=activity)
            sig_object.add_reference(
                referenced_uuid=sandbox_object.uuid,
                relationship_type="belongs-to",
            )
            objects.append(sig_object)

        technique_tags = []
        if mitre_attack_technique_id_to_tag:
            for technique in item.get("analysis.mitre_techniques", []):
                technique_id = technique.split(":")[0]
                if technique_id in mitre_attack_technique_id_to_tag:
                    technique_tags.append(mitre_attack_technique_id_to_tag[technique_id])

        contexa_tags = []
        for tag_name, tag_value in zip(
            item.get("research.tag.name", []),
            item.get("research.tag.value", []),
        ):
            contexa_tags.append(f"contexa:{tag_name}={tag_value}")

        for tag in itertools.chain(tags or [], technique_tags, contexa_tags):
            TagUtils.add_tag_to_object(file_object, tag)
        return objects



import abc
import collections
import datetime
import json
import os
import requests

from types import TracebackType
from typing import Any
from typing import Dict
from typing import Generator
from typing import Iterator
from typing import List
from typing import Optional
from typing import Type
from typing import TypeVar


class ObjectFactory:
    """Factory to decode complex objects from misp."""

    def __init__(self, event_data: Dict):
        """Constructor."""
        self._uuid_to_object = {x["uuid"]: x for x in event_data["Event"]["Object"]}
        self._reference_uuid_to_uuids = collections.defaultdict(set)
        for uuid, obj in self._uuid_to_object.items():
            if "ObjectReference" in obj:
                references = {x["referenced_uuid"] for x in obj["ObjectReference"]}
                for reference in references:
                    self._reference_uuid_to_uuids[reference].add(uuid)

    def _get_referencing_objects(
        self,
        source_object: Dict,
        referencing_object_name: str,
    ) -> List[Dict]:
        """Get all the objects with a given name referencing the source object."""
        ret = []
        for referencing_uuid in self._reference_uuid_to_uuids[source_object["uuid"]]:
            referencing_object = self._uuid_to_object[referencing_uuid]
            if referencing_object["name"] == referencing_object_name:
                ret.append(referencing_object)
        return ret

    def parse_sandbox_report(self, file_object: Dict) -> Dict:
        """Parse the sandbox report."""
        ret = {
            "task.portal_url": [],
            "task.score": [],
            "analysis.activities": [],
        }
        try:
            sandbox_report = self._get_referencing_objects(file_object, "sandbox-report")[0]
        except IndexError:
            return ret
        for attr in sandbox_report["Attribute"]:
            if attr["object_relation"] == "permalink":
                ret["task.portal_url"] = attr["value"]
            elif attr["object_relation"] == "score":
                ret["task.score"] = attr["value"]
        ret["analysis.activities"] = [
            attribute["value"]
            for sig_hit in self._get_referencing_objects(sandbox_report, "sb-signature")
            for attribute in sig_hit["Attribute"]
        ]
        return ret

    @classmethod
    def parse_techniques(cls, tags) -> List[str]:
        """Parse the MITRE techniques included."""
        ret = []
        for object_tag in tags:
            tag_name, tag_value = object_tag.split("=")
            tag_value = tag_value.strip("\"")
            if tag_name == "mitre-galaxy:mitre-attack-pattern":
                technique, technique_id = tag_value.split(" - ")
                ret.append(f"{technique_id}: {technique}")
        return ret

    @classmethod
    def parse_file_object(cls, file_object) -> Dict[str, Optional[str]]:
        """Parse the file object."""
        ret = {
            "file.md5": None,
            "file.sha1": None,
            "file.sha256": None,
            "file.name": None,
        }
        for attr in file_object["Attribute"]:
            if attr["type"] == "filename":
                ret["file.name"] = attr["value"]
            elif attr["type"] == "sha1":
                ret["file.sha1"] = attr["value"]
            elif attr["type"] == "md5":
                ret["file.md5"] = attr["value"]
            elif attr["type"] == "sha256":
                ret["file.sha256"] = attr["value"]
        return ret


class FeedParser(abc.ABC):
    """Abstract class providing utility methods to all the parsers."""

    DEFAULT_FMT = "%Y-%m-%d %H:%M:%S"

    FILE_INDICATOR_TYPES = frozenset([
        "md5",
        "sha1",
        "sha256",
    ])

    NETWORK_INDICATOR_TYPES = frozenset([
        "domain",
        "ip",
        "url",
    ])

    FILE_OBJECT_TYPE = "file"

    NETWORK_OBJECT_TYPE = "network-profile"

    def __init__(self, event_data: Dict[str, Any]):
        """Constructor."""
        self._event_data = event_data
        self._uuid_to_object = {x["uuid"]: x for x in event_data["Event"]["Object"]}
        self._name_to_uuids = collections.defaultdict(list)
        for uuid, obj in self._uuid_to_object.items():
            self._name_to_uuids[obj["name"]].append(uuid)
        self._object_factory = ObjectFactory(event_data)

    @classmethod
    def _timestamp_to_date_str(cls, timestamp: float, fmt: str = None) -> str:
        """Convert a timestamp into a date string."""
        date_object = datetime.datetime.fromtimestamp(int(timestamp))
        return date_object.strftime(fmt or cls.DEFAULT_FMT)

    @classmethod
    def _get_tags_from_object(cls, object_data: Dict[str, Any]) -> List[str]:
        """Return the tags assigned to an object."""
        tags = set([])
        for attribute in object_data["Attribute"]:
            tags.update([x["name"] for x in attribute.get("Tag", [])])
        return sorted(tags)

    @classmethod
    def get_tags_from_event(cls, event_data: Dict[str, Any]) -> List[str]:
        """Get all the tags assigned to an event."""
        return sorted(set(x["name"] for x in event_data["Event"].get("Tag", [])))

    @abc.abstractmethod
    def __iter__(self) -> Iterator[Dict[str, Any]]:
        """Iterate."""

    def __enter__(self) -> Iterator[Dict[str, Any]]:
        """Return the iterator."""
        return iter(self)

    def __exit__(
        self,
        exc_type: Optional[Type[BaseException]],
        exc_value: Optional[BaseException],
        traceback: Optional[TracebackType],
    ) -> None:
        """Exit the context."""


BaseFeedParserSubType = TypeVar("BaseFeedParserSubType", bound=FeedParser)


class IndicatorEventFeedParser(FeedParser):
    """Parser able to read feeds made of indicators."""

    def _process_object(self, uuid: str) -> Generator[Dict[str, Any], None, None]:
        """Parse an object and return the item representation."""
        object_data = self._uuid_to_object[uuid]
        object_tags = self._get_tags_from_object(object_data)
        for attribute in object_data["Attribute"]:
            if attribute["type"] in self.FILE_INDICATOR_TYPES | self.NETWORK_INDICATOR_TYPES:
                timestamp = self._timestamp_to_date_str(self._event_data["Event"]["timestamp"])
                yield {
                    "tags": object_tags,
                    "timestamp": timestamp,
                    "event_uuid": self._event_data["Event"]["uuid"],
                    "object_uuid": uuid,
                    "attribute_uuid": attribute["uuid"],
                    "attribute_type": attribute["type"],
                    "attribute_value": attribute["value"],
                }

    def __iter__(self) -> Generator[Dict[str, Any], None, None]:
        """Implement interface."""
        for obj in self._uuid_to_object.values():
            if obj["name"] in frozenset([self.FILE_OBJECT_TYPE, self.NETWORK_OBJECT_TYPE]):
                yield from self._process_object(obj["uuid"])


class TelemetryEventFeedParser(FeedParser):
    """Parser able to read feeds made of telemetry items."""

    def _process_object(self, uuid: str) -> Generator[Dict[str, Any], None, None]:
        """Parse an object provided its uuids."""
        object_data = self._uuid_to_object[uuid]
        object_tags = self._get_tags_from_object(object_data)
        yield {
            "tags": object_tags,
            "techniques": self._object_factory.parse_techniques(object_tags),
            **self._object_factory.parse_sandbox_report(object_data),
            **self._object_factory.parse_file_object(object_data),
        }

    def __iter__(self) -> Generator[Dict[str, Any], None, None]:
        """Implement interface."""
        for name, uuids in self._name_to_uuids.items():
            if name == self.FILE_OBJECT_TYPE:
                for uuid in uuids:
                    yield from self._process_object(uuid)


class AbstractFeedConsumer(abc.ABC):
    """Abstract implementation of a consumer."""

    @abc.abstractmethod
    def load_manifest(self) -> Dict:
        """Return the manifest as a dictionary."""

    @abc.abstractmethod
    def load_event(self, event_uuid) -> Dict:
        """Load the event as a dictionary."""

    @classmethod
    def _infer_parser_class(cls, event_data: Dict) -> Type[BaseFeedParserSubType]:
        """Return the parser class able to read the feed."""
        for object_data in event_data["Event"]["Object"]:
            if object_data["name"] == "sandbox-report":
                return TelemetryEventFeedParser
        return IndicatorEventFeedParser

    def _get_event_uuids_since(self, since_timestamp: float = None) -> List[str]:
        """Read the manifest and return the event uuids matching the filter."""
        event_uuids = []
        for event_uuid, event_data in self.load_manifest().items():
            if not since_timestamp or event_data["timestamp"] > since_timestamp:
                event_uuids.append(event_uuid)
        return event_uuids

    def _get_events_since(self, date_object: datetime.datetime) -> List[Dict]:
        """Return list of event data objects."""
        event_uuids = self._get_event_uuids_since(date_object.timestamp())
        return [self.load_event(x) for x in event_uuids]

    def get_items_since(
        self,
        date_object: datetime.datetime,
        attribute_type: Optional[str] = None,
    ) -> List[Dict]:
        """Return the items contained in the feed."""
        ret = []
        for event_data in self._get_events_since(date_object):
            parser_class = self._infer_parser_class(event_data)
            with parser_class(event_data) as parser:
                for indicator in parser:
                    if not attribute_type or indicator["attribute_type"] == attribute_type:
                        ret.append(indicator)
        return ret


class LocalFeedConsumer(AbstractFeedConsumer):
    """Consumer using a local directory."""

    def load_manifest(self) -> Dict:
        """Implement interface."""
        with open(os.path.join(self._input_dir, "manifest.json"), "r") as f:
            return json.load(f)

    def load_event(self, event_uuid: str) -> Dict:
        """Implement interface."""
        with open(os.path.join(self._input_dir, f"{event_uuid}.json"), "r") as f:
            return json.load(f)

    def __init__(self, input_dir: str):
        """Constructor."""
        self._input_dir = input_dir


class RemoteFeedConsumer(AbstractFeedConsumer):
    """Consumer using a remote (HTTP) source."""

    def load_manifest(self) -> Dict:
        """Implement interface."""
        ret = requests.get(f"{self._base_url}/manifest.json")
        return ret.json()

    def load_event(self, event_uuid: str) -> Dict:
        """Implement interface."""
        ret = requests.get(f"{self._base_url}/{event_uuid}.json")
        return ret.json()

    def __init__(self, base_url: str):
        """Constructor."""
        self._base_url = base_url.rstrip("/")



import re

from typing import Optional


HEX_RE = re.compile(r"^[a-fA-F0-9]*$")


def get_hash_type(hash_value: str) -> Optional[str]:
    """Get the hash type."""
    if len(hash_value) == 32:
        return "md5" if HEX_RE.match(hash_value) else None
    elif len(hash_value) == 40:
        return "sha1" if HEX_RE.match(hash_value) else None
    elif len(hash_value) == 64:
        return "sha256" if HEX_RE.match(hash_value) else None
    else:
        return None


def print_dependency_error_and_raise(exception: Exception):
    """Print the error of the missing dependency and re-raise."""
    print("Only 'consumer.py' module is available without installing the 'pymisp' extra dependency")
    raise exception



#!/usr/bin/env python3
import argparse
import os
import pathlib
import sys

from feed_manager import translator
from feed_manager import generator


INDICATOR_MD5 = "a" * 32

INDICATOR_SHA1 = "b" * 40

INDICATOR_SHA256 = "c" * 64

INDICATOR_TAGS = [
    'misp-galaxy:malpedia="GootKit"',
    'misp-galaxy:threat-actor="Sofacy"',
]

TELEMETRY_ITEM = {
    "file.sha1": "a1f7670cd7da7e331db2d69f0855858985819873",
    "file.mime_type": "application/x-pe-app-32bit-i386",
    "analysis.label": "unclassified",
    "task.severity": "malicious",
    "file.llfile_type": "PeExeFile",
    "analysis.mitre_tactics": ["TA0005: Defense Evasion", "TA0007: Discovery"],
    "file.md5": "37840d4e937db0385b820d4019071540",
    "task.portal_url": (
        "https://user.lastline.com/portal#/analyst/task/"
        "30f48c17e9db002005baa7d440ca275a/overview"
    ),
    "file.magic": "Unknown",
    "task.uuid": "30f48c17e9db002005baa7d440ca275a",
    "source.origin": "API",
    "file.sha256": "492bfe8d2b1105ec4045f96913d38f98e30fe349ea50cc4aaa425ca289af2852",
    "customer.sector": "IT",
    "analysis.activities": [
        "Anomaly: AI detected possible malicious code reuse",
        "Evasion: Detecting the presence of AntiMalware Scan Interface (AMSI)",
        "Execution: Subject crash detected",
        "Signature: Potentially malicious application/program"
    ],
    "analysis.mitre_techniques": [
        "T1497: Virtualization/Sandbox Evasion"
    ],
    "task.score": 70,
    "customer.region": "AMER",
    "utc_timestamp": 1663568636000,
    "file.size": 990720,
    "file.name": None,
    "research.tag.name": [],
    "research.tag.value": [],
}


def main():
    """Simple script to generate a daily feed."""
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "-o",
        "--output-dir",
        dest="output_dir",
        type=str,
        default="./tmp/",
        help="the local feed",
    )
    args = parser.parse_args()

    # Test the indicator feed
    indicators_path = os.path.join(args.output_dir, "indicators")
    pathlib.Path(indicators_path).mkdir(parents=True, exist_ok=True)
    misp_object = translator.IndicatorTranslator.to_file_object(
        file_md5=INDICATOR_MD5,
        file_sha1=INDICATOR_SHA1,
        file_sha256=INDICATOR_SHA256,
        tags=INDICATOR_TAGS,
    )
    feed_generator = generator.DailyFeedGenerator(
        output_dir=indicators_path,
        feed_properties=generator.FeedProperties(
            title="Test indicators feed",
        ),
    )
    feed_generator.add_object_to_event(misp_object)
    feed_generator.flush_event()
    print(f"Daily feed of indicators written to: {indicators_path}")

    # Test the telemetry feed
    telemetry_path = os.path.join(args.output_dir, "telemetry")
    pathlib.Path(telemetry_path).mkdir(parents=True, exist_ok=True)
    misp_objects = translator.IndicatorTranslator.from_contexa_to_objects(
        item=TELEMETRY_ITEM,
        mitre_attack_technique_id_to_tag=None,
        include_sandbox_result=True,
        include_sandbox_activities=True,
    )
    feed_generator = generator.DailyFeedGenerator(
        output_dir=telemetry_path,
        feed_properties=generator.FeedProperties(
            title="Test telemetry feed",
        ),
    )
    for misp_object in misp_objects:
        feed_generator.add_object_to_event(misp_object)
    feed_generator.flush_event()
    print(f"Daily feed of telemetry objects written to: {telemetry_path}")

    return 0


if __name__ == "__main__":
    sys.exit(main())



import abc
import datetime
import hashlib
import json
import logging
import os

from typing import Dict
from typing import List
from typing import Optional
from typing import Tuple
from typing import Union

import feed_manager
from feed_manager import translator

try:
    import pymisp
except ImportError as ie:
    pymisp = None
    feed_manager.print_dependency_error_and_raise(ie)


class FeedProperties:
    """Class to represent the properties of a feed."""

    DEFAULT_ORG_NAME = "Organization"
    DEFAULT_ORG_UUID = "a6b0e426-8c54-4136-beb6-3c2bafac7c4b"
    DEFAULT_FEED_NAME = "Feed"

    @classmethod
    def create_organization(
        cls,
        name: str = DEFAULT_ORG_NAME,
        uuid: str = DEFAULT_ORG_UUID,
    ) -> pymisp.MISPOrganisation:
        organization = pymisp.MISPOrganisation()
        organization.from_dict(
            name=name,
            uuid=uuid,
        )
        return organization

    def __init__(
        self,
        analysis: str = "0",
        threat_level_id: str = "1",
        title: str = DEFAULT_FEED_NAME,
        published: bool = False,
        tags: Optional[List[Union[pymisp.MISPTag, str]]] = None,
        organization: Optional[pymisp.MISPOrganisation] = None,
    ):
        if not organization:
            organization = self.create_organization()
        if not tags:
            tags = []
        self.organization = organization
        self.analysis = analysis
        self.tags = [translator.TagUtils.validate_tag(x) for x in tags]
        self.threat_level_id = threat_level_id
        self.published = published
        self.title = title


class AbstractFeedGenerator(abc.ABC):
    """Abstract class for every feed generators."""

    DATE_FMT = "%Y-%m-%d"

    DATETIME_FMT = "%Y-%m-%d %H:%M:%S"

    def __init__(self, output_dir):
        """Constructor."""
        self._logger = logging.getLogger(__name__)
        self._output_dir = output_dir
        self._attribute_hashes = []
        self._manifest = {}

    @staticmethod
    def attribute_equals(attr1: pymisp.MISPAttribute, attr2: pymisp.MISPAttribute) -> bool:
        """Return whether two attributes are the same."""
        return (
            attr1.type == attr2.type and
            attr1.value == attr2.value and
            attr1.data == attr2.data
        )

    @staticmethod
    def tag_equals(tag1: pymisp.MISPTag, tag2: pymisp.MISPTag) -> bool:
        """Return whether two tags are the same."""
        return tag1.name == tag2.name

    @classmethod
    def object_equals(cls, obj1: pymisp.MISPObject, obj2: pymisp.MISPObject) -> bool:
        """Return whether two objects are the same."""
        obj1_attributes = sorted(obj1.attributes, key=lambda x: x.type)
        obj2_attributes = sorted(obj2.attributes, key=lambda x: x.type)
        if len(obj1_attributes) != len(obj2_attributes):
            return False
        for attr1, attr2 in zip(obj1_attributes, obj2_attributes):
            if not cls.attribute_equals(attr1, attr2):
                return False
        return True

    @classmethod
    def contains_attribute(
        cls,
        misp_event: pymisp.MISPEvent,
        attr_type: str,
        attr_value: str,
        **attr_data,
    ) -> bool:
        """Return whether the misp event contains a specific attribute."""
        fake_attribute = pymisp.MISPAttribute()
        fake_attribute.from_dict(
            type=attr_type,
            value=attr_value,
            data=attr_data,
        )
        for attr in misp_event.attributes:
            if cls.attribute_equals(fake_attribute, attr):
                return True
        return False

    @classmethod
    def contains_object(cls, misp_event: pymisp.MISPEvent, misp_object: pymisp.MISPObject) -> bool:
        """Return whether the misp event contains a specific object."""
        for obj in misp_event.objects:
            if cls.object_equals(obj, misp_object):
                return True
        return False

    @classmethod
    def contains_tag(cls, misp_event: pymisp.MISPEvent, misp_tag: pymisp.MISPTag) -> bool:
        for tag in misp_event.tags:
            if cls.tag_equals(tag, misp_tag):
                return True
        return False

    @abc.abstractmethod
    def add_object_to_event(self, misp_object: pymisp.MISPObject) -> bool:
        """Add object to the current event."""

    @abc.abstractmethod
    def add_tag_to_event(self, misp_tag: pymisp.MISPTag) -> bool:
        """Add tag to the current event."""

    @abc.abstractmethod
    def add_attribute_to_event(self, attr_type: str, attr_value: str, **attr_data) -> bool:
        """Add an attribute to the current event."""

    @abc.abstractmethod
    def flush_event(self, event: Optional[pymisp.MISPEvent] = None) -> None:
        """Flush the current event (if not specified)."""

    def _load_event(self, event_uuid: str) -> pymisp.MISPEvent:
        """Load an event give its uuid."""
        with open(os.path.join(self._output_dir, f"{event_uuid}.json"), "r") as f:
            event_dict = json.load(f)["Event"]
            event = pymisp.MISPEvent()
            event.from_dict(**event_dict)
            return event

    def _save_manifest(self) -> None:
        """Save the manifest to disk."""
        with open(os.path.join(self._output_dir, "manifest.json"), "w") as manifest_file:
            json.dump(self._manifest, manifest_file, indent=True)
        self._logger.debug("Manifest saved")

    def _load_manifest(self) -> Dict[str, Dict]:
        """Load the manifest."""
        manifest_path = os.path.join(self._output_dir, "manifest.json")
        with open(manifest_path, "r") as f:
            manifest = json.load(f)
        return manifest

    def _add_hash(self, event: pymisp.MISPEvent, attr_type: str, attr_value: str) -> None:
        """Take the attribute properties and add a hash."""
        _ = attr_type
        for frag in str(attr_value).split("|"):
            frag_hash = hashlib.md5(str(frag).encode("utf-8"), usedforsecurity=False).hexdigest()
            self._attribute_hashes.append([frag_hash, event.get("uuid")])

    def _save_hashes(self) -> None:
        """Save the collected hashes to disk."""
        with open(os.path.join(self._output_dir, "hashes.csv"), "a") as hash_file:
            for element in self._attribute_hashes:
                hash_file.write(f"{element[0]},{element[1]}\n")
        self._logger.debug("Hashes saved")
        self._attribute_hashes.clear()


class PeriodicFeedGenerator(AbstractFeedGenerator, abc.ABC):
    """A periodic feed generator that needs to be specialized further."""

    @classmethod
    @abc.abstractmethod
    def get_bucket(cls, date_obj: datetime.datetime) -> str:
        """Return the periodic bucket given the provided date object."""

    @classmethod
    def get_bucket_from_date_str(cls, event_date_str: str) -> str:
        """Get the bucket from the given date string."""
        try:
            date_obj = datetime.datetime.strptime(event_date_str, cls.DATETIME_FMT)
        except ValueError:
            date_obj = datetime.datetime.strptime(event_date_str, cls.DATE_FMT)
        return cls.get_bucket(date_obj)

    @classmethod
    def get_current_bucket(cls) -> str:
        """Get the current bucket (truncated datetime object)."""
        return cls.get_bucket(datetime.datetime.utcnow())

    def __init__(
        self,
        output_dir: str,
        feed_properties: Optional[FeedProperties] = None,
    ):
        """Constructor."""
        super(PeriodicFeedGenerator, self).__init__(output_dir)
        self._feed_properties = feed_properties or FeedProperties()
        try:
            self._manifest = self._load_manifest()
        except FileNotFoundError:
            self._logger.debug("Manifest not found, generating a new one")
            self._manifest = {}
            new_event = self._create_event(self.get_current_bucket())
            # flush new event for the first time and manifest
            self.flush_event(event=new_event)
            self._manifest.update(new_event.manifest)
            self._save_manifest()

        event_uuid, event_date_str = self._get_last_event_metadata()
        self._current_event_bucket = self.get_bucket_from_date_str(event_date_str)
        self._current_event = self._load_event(event_uuid)

    def add_object_to_event(self, misp_object: pymisp.MISPObject) -> bool:
        """Implement interface."""
        self._update_event_bucket()
        if self.contains_object(self._current_event, misp_object):
            return False
        self._current_event.add_object(misp_object)
        for attribute in misp_object.attributes:
            self._add_hash(self._current_event, attribute.type, attribute.value)
        return True

    def add_attribute_to_event(self, attr_type: str, attr_value: str, **attr_data) -> bool:
        """Implement interface."""
        self._update_event_bucket()
        if self.contains_attribute(self._current_event, attr_type, attr_value, **attr_data):
            return False
        self._current_event.add_attribute(attr_type, attr_value, **attr_data)
        self._add_hash(self._current_event, attr_type, attr_value)
        return True

    def add_tag_to_event(self, misp_tag: pymisp.MISPTag) -> bool:
        """Implement interface."""
        self._update_event_bucket()
        if self.contains_tag(self._current_event, misp_tag):
            return False
        self._current_event.add_tag(misp_tag)
        return True

    def flush_event(self, event: Optional[pymisp.MISPEvent] = None) -> None:
        """Implement interface."""
        if not event:
            event = self._current_event
        with open(os.path.join(self._output_dir, event.get("uuid") + ".json"), "w") as event_file:
            json.dump(event.to_feed(), event_file, indent=True)
        self._save_hashes()

    def _update_event_bucket(self) -> None:
        """Update the current bucket if needed."""
        event_bucket = self.get_current_bucket()
        if self._current_event_bucket != event_bucket:
            self._logger.debug(
                "New event bucket required (new=%s, old=%s)",
                event_bucket, self._current_event_bucket,
            )
            # flush previous event
            self.flush_event()
            # create new event
            self._current_event_bucket = event_bucket
            self._current_event = self._create_event(event_bucket)
            # flush new event for the first time and manifest
            self.flush_event()
            self._manifest.update(self._current_event.manifest)
            self._save_manifest()

    def _get_last_event_metadata(self) -> Tuple[str, str]:
        """Get the metadata related to the latest event."""
        dated_events = []
        for event_uuid, event_json in self._manifest.items():
            dated_events.append((
                event_json["date"],
                event_uuid,
                event_json["info"],
            ))
        # Sort by date then by event name
        dated_events.sort(key=lambda k: (k[0], k[2], k[1]), reverse=True)
        return dated_events[0][1], dated_events[0][0]

    def _create_event(self, event_bucket: str) -> pymisp.MISPEvent:
        """Create an even in the given bucket."""
        event = pymisp.MISPEvent()
        event.from_dict(**{
            "id": str(len(self._manifest) + 1),
            "info": f"{self._feed_properties.title} ({event_bucket})",
            "date": datetime.datetime.utcnow().strftime(self.DATETIME_FMT),
            "analysis": self._feed_properties.analysis,
            "threat_level_id": self._feed_properties.threat_level_id,
            "published": self._feed_properties.published,
        })
        for tag in self._feed_properties.tags:
            event.add_tag(tag)
        event.Orgc = self._feed_properties.organization
        return event


class DailyFeedGenerator(PeriodicFeedGenerator):
    """A feed generator that creates a different event every day."""

    BUCKET_FMT = "%Y-%m-%d"

    @classmethod
    def get_bucket(cls, date_obj: datetime.datetime) -> str:
        """Implement interface."""
        return date_obj.replace(
            hour=0,
            minute=0,
            second=0,
            microsecond=0,
        ).strftime(cls.BUCKET_FMT)
